<h2><center>kubernetes 资源管理</center></h2>

------

## 一：资源管理

### 1. 资源管理介绍

​	在`kubernetes`中，所有的内容都抽象为资源，用户需要通过操作资源来管理`kubernetes`。

- `kubernetes`的本质上就是一个集群系统，用户可以在集群中部署各种服务，所谓的部署服务，其实就是在`kubernetes`集群中运行一个个的容器，并将指定的程序跑在容器中。
- `kubernetes`的最小管理单元是`pod`而不是容器，所以只能将容器放在`Pod`中，而`kubernetes`一般也不会直接管理`Pod`，而是通过`Pod`控制器来管理`Pod`的。
- `Pod`可以提供服务之后，就要考虑如何访问`Pod`中服务，`kubernetes`提供了`Service`资源实现这个功能。
- 当然，如果`Pod`中程序的数据需要持久化，`kubernetes`还提供了各种存储系统。

![](http://182.92.143.66:40072/directlink/img/kubernetes/images-202504140007.png)

### 2. 资源管理方式

- 命令式对象管理：直接使用命令去操作`kubernetes`资源

  ```bash
  kubectl run nginx-pod --image=nginx:1.17.1 --port=80
  ```

- 命令式对象配置：通过命令配置和配置文件去操作`kubernetes`资源

  ```bash
  kubectl create/patch -f nginx-pod.yaml
  ```

- 声明式对象配置：通过`apply`命令和配置文件去操作`kubernetes`资源

  ```bash
  kubectl apply -f nginx-pod.yaml
  ```

|      类型      | 操作对象 | 适用环境 |      优点      |               缺点               |
| :------------: | :------: | :------: | :------------: | :------------------------------: |
| 命令式对象管理 |   对象   |   测试   |      简单      | 只能操作活动对象，无法审计、跟踪 |
| 命令式对象配置 |   文件   |   开发   | 可以审计、跟踪 |  项目大时，配置文件多，操作麻烦  |
| 声明式对象配置 |   目录   |   开发   |  支持目录操作  |        意外情况下难以调试        |

#### 1. 命令式对象管理

**`kubectl`命令**

`kubectl`是`kubernetes`集群的命令行工具，通过它能够对集群本身进行管理，并能够在集群上进行容器化应用的安装部署。`kubectl`命令的语法如下：

```bash
kubectl [command] [type] [name] [flags]
```

- `comand`：指定要对资源执行的操作，例如`create`、`get`、`delete`
- `type`：指定资源类型，比如`deployment`、`pod`、`service`
- `name`：指定资源的名称，名称大小写敏感
- `flags`：指定额外的可选参数

```bash
# 查看所有pod
kubectl get pod 

# 查看某个pod
kubectl get pod pod_name

# 查看某个pod,以yaml格式展示结果
kubectl get pod pod_name -o yaml
```

**资源类型**

`kubernetes`中所有的内容都抽象为资源，可以通过下面的命令进行查看：

```bash
kubectl api-resources
```

经常使用的资源有下面这些：

|    资源分类     |          资源名称          |   缩写   |     资源作用      |
| :-------------: | :------------------------: | :------: | :---------------: |
|  集群级别资源   |          `nodes`           |   `no`   |   集群组成部分    |
|  `namespaces`   |            `ns`            | 隔离Pod  |                   |
|    `pod`资源    |           `pods`           |   `po`   |     装载容器      |
| `pod`资源控制器 |  `replicationcontrollers`  |   `rc`   |   控制`pod`资源   |
|                 |       `replicasets`        |   `rs`   |   控制`pod`资源   |
|                 |       `deployments`        | `deploy` |   控制`pod`资源   |
|                 |        `daemonsets`        |   `ds`   |   控制`pod`资源   |
|                 |           `jobs`           |          |   控制`pod`资源   |
|                 |         `cronjobs`         |   `cj`   |   控制`pod`资源   |
|                 | `horizontalpodautoscalers` |  `hpa`   |   控制`pod`资源   |
|                 |       `statefulsets`       |  `sts`   |   控制`pod`资源   |
|  服务发现资源   |         `services`         |  `svc`   | 统一`pod`对外接口 |
|                 |         `ingress`          |  `ing`   | 统一`pod`对外接口 |
|    存储资源     |    `volumeattachments`     |          |       存储        |
|                 |    `persistentvolumes`     |   `pv`   |       存储        |
|                 |  `persistentvolumeclaims`  |  `pvc`   |       存储        |
|                 |        `configmaps`        |   `cm`   |       配置        |
|                 |         `secrets`          |          |       配置        |

**操作**

`kubernetes`允许对资源进行多种操作，可以通过`–help`查看详细的操作命令

```bash
kubectl --help
```

经常使用的操作有下面这些：

|  命令分类  |      命令      |           翻译            |           命令作用           |
| :--------: | :------------: | :-----------------------: | :--------------------------: |
|  基本命令  |    `create`    |           创建            |         创建一个资源         |
|            |     `edit`     |           编辑            |         编辑一个资源         |
|            |     `get`      |           获取            |         获取一个资源         |
|            |    `patch`     |           更新            |         更新一个资源         |
|            |    `delete`    |           删除            |         删除一个资源         |
|            |   `explain`    |           解释            |         展示资源文档         |
| 运行和调试 |     `run`      |           运行            |  在集群中运行一个指定的镜像  |
|            |    `expose`    |           暴露            |      暴露资源为Service       |
|            |   `describe`   |           描述            |       显示资源内部信息       |
|            |     `logs`     | 日志输出容器在pod中的日志 |    输出容器在pod中的日志     |
|            |    `attach`    |   缠绕进入运行中的容器    |       进入运行中的容器       |
|            |     `exec`     |   执行容器中的一个命令    |     执行容器中的一个命令     |
|            |      `cp`      |           复制            |      在Pod内外复制文件       |
|            |   `rollout`    |         首次展示          |        管理资源的发布        |
|            |    `scale`     |           规模            |     扩（缩）容Pod的数量      |
|            |  `autoscale`   |         自动调整          |     自动调整`Pod`的数量      |
|  高级命令  |    `apply`     |           `rc`            |    通过文件对资源进行配置    |
|            |    `label`     |           标签            |       更新资源上的标签       |
|  其他命令  | `cluster-info` |         集群信息          |         显示集群信息         |
|            |   `version`    |           版本            | 显示当前Server和Client的版本 |

下面以一个`namespace` / `pod`的创建和删除简单演示下命令的使用：

```bash
# 创建一个namespace
[root@master ~]# kubectl create namespace dev
namespace/dev created

# 获取namespace
[root@master ~]# kubectl get ns
NAME              STATUS   AGE
default           Active   21h
dev               Active   21s
kube-node-lease   Active   21h
kube-public       Active   21h
kube-system       Active   21h

# 在此namespace下创建并运行一个nginx的Pod
[root@master ~]# kubectl run pod --image=nginx:latest -n dev
kubectl run --generator=deployment/apps.v1 is DEPRECATED and will be removed in a future version. Use kubectl run --generator=run-pod/v1 or kubectl create instead.
deployment.apps/pod created

# 查看新创建的pod
[root@master ~]# kubectl get pod -n dev
NAME  READY   STATUS    RESTARTS   AGE
pod   1/1     Running   0          21s

# 删除指定的pod
[root@master ~]# kubectl delete pod pod-864f9875b9-pcw7x
pod "pod" deleted

# 删除指定的namespace
[root@master ~]# kubectl delete ns dev
namespace "dev" deleted
```

#### 2. 命令式对象配置

命令式对象配置就是使用命令配合配置文件一起来操作`kubernetes`资源。

1）创建一个`nignxpod.yaml`，内容如下：

```yml
# 定义 Namespace（第一部分）
apiVersion: v1
kind: Namespace
metadata:
  name: dev

# 用三个短横线分隔多个资源（第二部分）
---

# 定义 Pod（第二部分）
apiVersion: v1
kind: Pod
metadata:
  name: nginxpod
  namespace: dev
spec:
  containers:
  - name: nginx-containers
    image: nginx:latest
```

2）执行`create`命令，创建资源：

```bash
[root@master ~]# kubectl create -f nginxpod.yaml
namespace/dev created
pod/nginxpod created
```

此时发现创建了两个资源对象，分别是`namespace`和`pod`

3）执行`get`命令，查看资源：

```bash
[root@master ~]#  kubectl get -f nginxpod.yaml
NAME            STATUS   AGE
namespace/dev   Active   18s

NAME            READY   STATUS    RESTARTS   AGE
pod/nginxpod    1/1     Running   0          17s
```

这样就显示了两个资源对象的信息

4）执行`delete`命令，删除资源：

```bash
[root@master ~]# kubectl delete -f nginxpod.yaml
namespace "dev" deleted
pod "nginxpod" deleted
```

此时发现两个资源对象被删除了

**总结：**

​	命令式对象配置的方式操作资源，可以简单的认为：命令  +  yaml配置文件（里面是命令需要的各种参数）

#### 3. 声明式对象配置

声明式对象配置跟命令式对象配置很相似，但是它只有一个命令`apply`。

```bash
# 首先执行一次kubectl apply -f yaml文件，发现创建了资源
[root@master ~]#  kubectl apply -f nginxpod.yaml
namespace/dev created
pod/nginxpod created

# 再次执行一次kubectl apply -f yaml文件，发现说资源没有变动
[root@master ~]#  kubectl apply -f nginxpod.yaml
namespace/dev unchanged
pod/nginxpod unchanged
```

**总结：**

其实声明式对象配置就是使用`apply`描述一个资源最终的状态（在`yaml`中定义状态）

使用`apply`操作资源：

- 如果资源不存在，就创建，相当于`kubectl create`
- 如果资源已存在，就更新，相当于`kubectl patch`

**扩展：`kubectl`可以在`node`节点上运行吗？**

​	`kubectl`的运行是需要进行配置的，它的配置文件是`$HOME/.kube`，如果想要在`node`节点运行此命令，需要将`master`上的`.kube`文件复制到`node`节点上，即在`master`节点上执行下面操作：

```bash
scp  -r  HOME/.kube   node1: HOME/
```

**使用推荐：三种方式应该怎么用？**

- 创建/更新资源 使用声明式对象配置`kubectl apply -f xxx.yaml`
- 删除资源 使用命令式对象配置`kubectl delete -f xxx.yaml`
- 查询资源 使用命令式对象管理`kubectl get(describe) 资源名称`